#include "slideshow.h"
#include "../stub/ifapp.h"
#include <qapplication.h>
#include <qpushbutton.h>
#include <kconfig.h>
#include <kglobal.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <stdlib.h>


KIFSlideShow::KIFSlideShow(const QStringList &fileList, int delay,
                           bool maxpect, bool loop, QWidget *parent,
                           const char *name)
    : QWidget(parent, name, WType_Popup | WDestructiveClose)
{
    connect(&t, SIGNAL(timeout()), this, SLOT(slotTimer()));
    setBackgroundMode(NoBackground);

    // nasty casts for the GC because we want it global to the class but
    // can't include Xlib.h because of many Qt conflicts, so need to use
    // a void ptr
    gc = malloc(sizeof(GC));
    *((GC*)gc) = XCreateGC(x11Display(),
                           RootWindow(x11Display(), x11Screen()), 0, 0 );

    KConfig *config = KGlobal::config();
    config->setGroup("UISettings");
    // alloc color for lowcolor displays
    c = new QColor(config->readColorEntry("FullScreenColor", &Qt::white));
    XSetForeground(x11Display(), *((GC*)gc), c->pixel());

    max = maxpect;
    looping = loop;
    time = delay;
    list = fileList;
    move(0, 0);
    resize(QApplication::desktop()->size());
    it = list.begin();
    slotTimer();
    show();
    t.start(time*1000);
}

KIFSlideShow::~KIFSlideShow()
{
    qWarning("Slideshow finished");
    XFreeGC(x11Display(), *((GC*)gc));
    free(gc);
    delete c;
}


void KIFSlideShow::slotTimer()
{
    t.stop();
    if(it == list.end()){
        if(looping)
            it = list.begin();
        else{
            close();
            return;
        }
    }
    kifLoadImage(img, (*it));
    if(!img.isNull()){
        if(img.width() > width() || img.height() > height()){
            int w = img.width();
            int h = img.height();
            if(w > width()){
                float percent = ((float)width())/w;
                w = (int)(w*percent);
                h = (int)(h*percent);
            }
            if(h > height()){
                float percent = ((float)height())/h;
                w = (int)(w*percent);
                h = (int)(h*percent);
            }
            img = img.smoothScale(w, h);
        }
        else if(max && img.width() < width() && img.height() < height()){
            int w = img.width();
            int h = img.height();
            float percentW, percentH;

            // decide if to scale horizontally or vertically
            percentW = percentH = 0.0;
            if(img.width() < width())
                percentW = ((float)width())/w;
            if(img.height() < height())
                percentH = ((float)height())/h;
            // can we scale width to screensize w/out making height offscreen
            if(percentW && ((int)(h * percentW)) <= height()){
                w = (int)(w*percentW);
                h = (int)(h*percentW);
            }
            // if not try to scale horizontally w/out making width offscreen
            else if(percentH && ((int)(w * percentH)) <= width()){
                w = (int)(w*percentH);
                h = (int)(h*percentH);
            }
            img = img.smoothScale(w, h);
        }
        pix.convertFromImage(img);
    }
    else
        pix.resize(0, 0);


    repaint(false);
    ++it;
    t.start(time*1000);
}


void KIFSlideShow::paintEvent(QPaintEvent *ev)
{
    if(pix.isNull()){
        XFillRectangle(x11Display(), winId(),  *((GC*)gc), 0, 0, width(),
                       height());
        return;
    }
    QRect pixRect(0, 0, pix.width(), pix.height());
    QRect drawRect(ev->rect());
    if(pix.width() < width()){
        pixRect.setLeft(((width()-pix.width())/2)-1);
        pixRect.setWidth(pix.width()); // need to reset
    }
    if(pix.height() < height()){
        pixRect.setTop(((height()-pix.height())/2)-1);
        pixRect.setHeight(pix.height());
    }
    pixRect = pixRect.intersect(drawRect);
    if(pix.mask())
        XFillRectangle(x11Display(), winId(), *((GC*)gc), pixRect.x(),
                       pixRect.y(), drawRect.width(), drawRect.height());
    bitBlt(this, pixRect.topLeft(), &pix, drawRect, Qt::CopyROP);
    // fill in the edges
    int rects = 0;
    XRectangle rectangles[4];
    if(drawRect.top() < pixRect.top()){
        rectangles[rects].x = drawRect.x();
        rectangles[rects].y = drawRect.x();
        rectangles[rects].width = drawRect.width();
        rectangles[rects].height =  pixRect.top()-drawRect.top();
        ++rects;
    }
    if(drawRect.left() < pixRect.left()){
        rectangles[rects].x = drawRect.x();
        rectangles[rects].y = pixRect.top()-drawRect.top();
        rectangles[rects].width = pixRect.left()-drawRect.left();
        rectangles[rects].height = drawRect.height()-pixRect.top();
        ++rects;
    }
    if(drawRect.right() > pixRect.right()){
        rectangles[rects].x = pixRect.right()+1;
        rectangles[rects].y = pixRect.top()-drawRect.top();
        rectangles[rects].width = drawRect.right()-pixRect.right();
        rectangles[rects].height = drawRect.height()-pixRect.top();
        ++rects;
    }
    if(drawRect.bottom() > pixRect.bottom()){
        rectangles[rects].x = pixRect.left();
        rectangles[rects].y = pixRect.bottom()+1;
        rectangles[rects].width = pixRect.width();
        rectangles[rects].height = drawRect.bottom()-pixRect.bottom();
        ++rects;
    }
    if(rects)
        XFillRectangles(x11Display(), winId(), *((GC*)gc), rectangles, rects);

}

void KIFSlideShow::closeEvent(QCloseEvent *ev)
{
    if(t.isActive())
        t.stop();
    pix.resize(0, 0);
    ev->accept();
}

void KIFSlideShow::mousePressEvent(QMouseEvent *ev)
{
    if(ev->button() == LeftButton){
        close();
    }
}

